(*--------------------------------------------------------------------------*)
(*        KeyPressed --- Return TRUE if key pressed                         *)
(*--------------------------------------------------------------------------*)

FUNCTION KeyPressed : BOOLEAN;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*     Function:  KeyPressed                                                *)
(*                                                                          *)
(*     Purpose:   Return TRUE if key pressed                                *)
(*                                                                          *)
(*     Calling sequence:                                                    *)
(*                                                                          *)
(*        KeyHit := KeyPressed;                                             *)
(*                                                                          *)
(*           KeyHit --- If key hit, return TRUE else FALSE.                 *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

VAR
   Regs : Registers;

BEGIN (* KeyPressed *)

   Regs.AH := 11;
   MSDOS( Regs );

   KeyPressed := ( Regs.AL = 255 );

END   (* KeyPressed *);

(*--------------------------------------------------------------------------*)
(*     TimeOfDayString --- Return current time of day as string             *)
(*--------------------------------------------------------------------------*)

FUNCTION TimeOfDayString : AnyStr;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*     Function:  TimeOfDayString                                           *)
(*                                                                          *)
(*     Purpose:   Return current time of day as string                      *)
(*                                                                          *)
(*     Calling sequence:                                                    *)
(*                                                                          *)
(*        Tstring := TimeOfDayString : AnyStr;                              *)
(*                                                                          *)
(*           Tstring  --- Resultant 'HH:MM am/pm' form of time              *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

VAR
   Hours   : WORD;
   Minutes : WORD;
   Seconds : WORD;
   SecHun  : WORD;
   SH      : STRING[2];
   SM      : STRING[2];
   AmPm    : STRING[2];

BEGIN (* TimeOfDayString *)

   GetTime( Hours, Minutes, Seconds, SecHun );

   Adjust_Hour( Hours , AmPm );

   STR( Hours  :2, SH );
   STR( Minutes:2, SM );

   IF SM[1] = ' ' THEN SM[1] := '0';

   TimeOfDayString := SH + ':' + SM + ' ' + AmPm;

END   (* TimeOfDayString *);

(*--------------------------------------------------------------------------*)
(*             DateString  --- Return current date in string form           *)
(*--------------------------------------------------------------------------*)

FUNCTION DateString : AnyStr;

(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*     Function:  DateString                                                *)
(*                                                                          *)
(*     Purpose:   Returns current date in string form                       *)
(*                                                                          *)
(*     Calling sequence:                                                    *)
(*                                                                          *)
(*        Dstring := DateString: AnyStr;                                    *)
(*                                                                          *)
(*           Dstring     --- Resultant string form of date                  *)
(*                                                                          *)
(*     Calls:  GetDate                                                      *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

VAR
   SDay:           STRING[2];
   SYear:          STRING[4];
   Month:          WORD;
   Day:            WORD;
   Year:           WORD;
   DayOfWeek:      WORD;

BEGIN (* DateString *)
                                   (* Date function *)

   GetDate( Year, Month, Day, DayOfWeek );

                                   (* Convert date to string *)

   STR( ( Year - 1900 ):2  , SYear  );
   STR( Day :2  , SDay   );

   DateString := SDay + '-' + Month_Names[ Month ] + '-' + SYear;

END   (* DateString *);

(*----------------------------------------------------------------------*)
(*            Open_File --- Open untyped file for processing            *)
(*----------------------------------------------------------------------*)

PROCEDURE Open_File(     FileName : AnyStr;
                     VAR AFile    : FILE;
                     VAR File_Pos : LONGINT;
                     VAR Error    : INTEGER );

(*----------------------------------------------------------------------*)
(*                                                                      *)
(*    Procedure: Open_File                                              *)
(*                                                                      *)
(*    Purpose:   Opens untyped file (of byte) for input                 *)
(*                                                                      *)
(*    Calling sequence:                                                 *)
(*                                                                      *)
(*       Open_File(     FileName : AnyStr;                              *)
(*                  VAR AFile    : FILE;                                *)
(*                  VAR File_Pos : LONGINT;                             *)
(*                  VAR Error    : INTEGER );                           *)
(*                                                                      *)
(*          FileName --- Name of file to open                           *)
(*          AFile    --- Associated file variable                       *)
(*          File_Pos --- Initial byte offset in file (always set to 0)  *)
(*          Error    --- =  0:  Open went OK.                           *)
(*                       <> 0:  Open failed.                            *)
(*                                                                      *)
(*----------------------------------------------------------------------*)

BEGIN (* Open_File *)
                                   (* Try opening file.  Access       *)
                                   (* is essentially as file of byte. *)
   FileMode := 0;

   ASSIGN( AFile , FileName );
   RESET ( AFile , 1 );

   FileMode := 2;
                                   (* Check if open went OK or not *)
   IF ( IOResult <> 0 ) THEN
      Error := Open_Error
   ELSE
      Error := 0;
                                   (* We are at beginning of file *)
   File_Pos := 0;

END   (* Open_File *);

(*----------------------------------------------------------------------*)
(*              Close_File --- Close an unytped file                    *)
(*----------------------------------------------------------------------*)

PROCEDURE Close_File( VAR AFile : FILE );

(*----------------------------------------------------------------------*)
(*                                                                      *)
(*    Procedure: Close_File                                             *)
(*                                                                      *)
(*    Purpose:   Closes untyped file                                    *)
(*                                                                      *)
(*    Calling sequence:                                                 *)
(*                                                                      *)
(*       Close_File( VAR AFile : FILE );                                *)
(*                                                                      *)
(*          AFile    --- Associated file variable                       *)
(*                                                                      *)
(*----------------------------------------------------------------------*)

BEGIN (* Close_File *)
                                   (* Close the file *)
   CLOSE( AFile );
                                   (* Clear error flag *)
   IF ( IOResult <> 0 ) THEN;

END   (* Close_File *);

(*----------------------------------------------------------------------*)
(*          Quit_Found --- Check if ^C hit on keyboard                  *)
(*----------------------------------------------------------------------*)

FUNCTION QuitFound : BOOLEAN;

(*----------------------------------------------------------------------*)
(*                                                                      *)
(*    Function:  Quit_Found                                             *)
(*                                                                      *)
(*    Purpose:   Determines if keyboard input is ^C                     *)
(*                                                                      *)
(*    Calling sequence:                                                 *)
(*                                                                      *)
(*       Quit := Quit_Found : BOOLEAN;                                  *)
(*                                                                      *)
(*          Quit  --- TRUE if ^C typed at keyboard.                     *)
(*                                                                      *)
(*    Remarks:                                                          *)
(*                                                                      *)
(*       The cataloguing process can be halted by hitting ^C at the     *)
(*       keyboard.  This routine is called when Find_Files notices that *)
(*       keyboard input is waiting.  If ^C is found, then cataloguing   *)
(*       stops at the next convenient breakpoint.  The global variable  *)
(*       User_Break indicates that a ^C was found.                      *)
(*                                                                      *)
(*----------------------------------------------------------------------*)

VAR
   Ch : CHAR;

BEGIN (* QuitFound *)
                                   (* Character was hit -- read it *)
   READ( Ch );
                                   (* If it is a ^C, set User_Break *)
                                   (* so we halt at next convenient *)
                                   (* location.                     *)

   User_Break := User_Break OR ( Ch = ^C );
   QuitFound  := User_Break;
                                   (* Purge anything else in keyboard *)
                                   (* buffer                          *)
   WHILE( KeyPressed ) DO
      READ( Ch );

END   (* QuitFound *);

(*----------------------------------------------------------------------*)
(*           Check_Entry_Spec --- Check if entry spec is legitimate     *)
(*----------------------------------------------------------------------*)

PROCEDURE Check_Entry_Spec(     Entry_Spec     : AnyStr;
                            VAR Entry_Name     : String8;
                            VAR Entry_Ext      : String3;
                            VAR Use_Entry_Spec : BOOLEAN );

(*----------------------------------------------------------------------*)
(*                                                                      *)
(*    Procedure: Check_Entry_Spec                                       *)
(*                                                                      *)
(*    Purpose:   Check_Entry_Spec                                       *)
(*                                                                      *)
(*    Calling sequence:                                                 *)
(*                                                                      *)
(*       Check_Entry_Spec(     Entry_Spec     : AnyStr;                 *)
(*                         VAR Entry_Name     : String8;                *)
(*                         VAR Entry_Ext      : String3;                *)
(*                         VAR Use_Entry_Spec : BOOLEAN );              *)
(*                                                                      *)
(*          Entry_Spec     --- The wildcard for .ARC/.LBR contents.     *)
(*          Entry_Name     --- Output 8-char name part of wildcard      *)
(*          Entry_Ext      --- Output 3-char extension part of wildcard *)
(*          Use_Entry_Spec --- TRUE if Entry_Spec legitimate and not    *)
(*                             equivalent to a "get all entries."       *)
(*                                                                      *)
(*    Remarks:                                                          *)
(*                                                                      *)
(*       This routine splits the original wildcard specification into   *)
(*       two parts:  one corresponding to the name portion, and the     *)
(*       other the extension portion.  "*" (match string) characters    *)
(*       are converted to an appropriate series of "?" (match one char) *)
(*       characters.                                                    *)
(*                                                                      *)
(*----------------------------------------------------------------------*)

VAR
   ISpec : INTEGER;
   IDot  : INTEGER;
   LSpec : INTEGER;
   IOut  : INTEGER;
   QExt  : BOOLEAN;

BEGIN (* Check_Entry_Spec *)
                                   (* Initialize name, extension *)
                                   (* portion of wildcard        *)
   Entry_Name := '????????';
   Entry_Ext  := '???';
                                   (* IOut points to name/ext position *)
   IOut  := 0;
                                   (* ISpec points to wildcard position *)
   ISpec := 0;
                                   (* Get length of wildcard *)

   LSpec := Min( LENGTH( Entry_Spec ) , 12 );

                                   (* See if '.' appears in Entry_Spec.  *)
                                   (* If not, assume one after name part *)
                                   (* of wildcard.                       *)

   IDot := POS( '.' , Entry_Spec );
   IF ( IDot = 0 ) THEN
      IDot := 9;
                                   (* Point to first character in wildcard *)
   ISpec := 1;
                                   (* We start storing in name, not extension *)
   QExt  := FALSE;
                                   (* Loop over characters in wildcard *)

   WHILE( ISpec <= LSpec ) DO
      BEGIN
                                   (* Handle '.', '*', '?' specially; copy *)
                                   (* rest directly to either name or      *)
                                   (* extension portion of wildcard.       *)

         CASE Entry_Spec[ISpec] OF

            '.': BEGIN
                    IOut := 0;
                    QExt := TRUE;
                 END;
            '*': IF QExt THEN
                    ISpec := 12
                 ELSE
                    ISpec := PRED( IDot );
            '?': INC( IOut );
            ELSE BEGIN
                    INC( IOut );
                    IF QExt THEN
                       Entry_Ext[IOut]  := Entry_Spec[ISpec]
                    ELSE
                       Entry_Name[IOut] := Entry_Spec[ISpec]
                 END;

         END;
                                   (* Point to next character in wildcard. *)
         INC( ISpec );

      END;
                                   (* If wildcard turns out to be a  *)
                                   (* 'match anything' spec, don't   *)
                                   (* bother with any matching later *)
                                   (* on.                            *)

   Use_Entry_Spec := ( Entry_Name <> '????????' ) OR
                     ( Entry_Ext  <> '???'      );

END   (* Check_Entry_Spec *);

(*----------------------------------------------------------------------*)
(*     Entry_Matches --- Check if given file name matches entry spec    *)
(*----------------------------------------------------------------------*)

FUNCTION Entry_Matches( FileName : AnyStr ) : BOOLEAN;

(*----------------------------------------------------------------------*)
(*                                                                      *)
(*    Function:  Entry_Matches                                          *)
(*                                                                      *)
(*    Purpose:   Entry_Matches                                          *)
(*                                                                      *)
(*    Calling sequence:                                                 *)
(*                                                                      *)
(*       Matches := Entry_Matches( VAR FileName : AnyStr ) : BOOLEAN;   *)
(*                                                                      *)
(*          FileName --- name of file to check against entry spec       *)
(*          Matches  --- set TRUE if FileName matches global            *)
(*                       entry spec contained in 'Entry_Spec'.          *)
(*                                                                      *)
(*----------------------------------------------------------------------*)

VAR
   IDot  : INTEGER;
   IPos  : INTEGER;
   Match : BOOLEAN;
   FName : STRING[8];
   FExt  : STRING[3];
   LName : INTEGER;

BEGIN (* Entry_Matches *)
                                   (* Assume match found to start. *)
   Match := TRUE;
                                   (* Initialize wildcard form of  *)
                                   (* file name and extension.     *)
   FName := '????????';
   FExt  := '???';
                                   (* Get length of filename *)
   LName := LENGTH( FileName );
                                   (* See if '.' appears in filename.    *)
   IDot := POS( '.' , FileName );
                                   (* Move name field to wildcard pattern *)
   IF ( IDot > 0 ) THEN
      BEGIN
         MOVE( FileName[1],      FName[1], IDot  - 1    );
         MOVE( FileName[IDot+1], FExt [1], LName - IDot )
      END
   ELSE
      MOVE( FileName[1], FName[1], LName );

                                   (* IPos has position in name portion *)
   IPos := 0;
                                   (* Try matching name portion of file name *)
                                   (* with wildcard for name portion.        *)
   REPEAT
      INC( IPos );
      IF ( Entry_Name[IPos] <> '?' ) THEN
         Match := Match AND ( FName[IPos] = Entry_Name[IPos] );
   UNTIL ( NOT Match ) OR ( IPos = 8 );

                                   (* IPos has position in extension portion *)
   IPos := 0;
                                   (* Try matching extension portion of file *)
                                   (* name with wildcard for extension       *)
                                   (* portion.  Unnecessary if name portions *)
                                   (* didn't match.                          *)
   IF Match THEN
      REPEAT
         INC( IPos );
         IF ( Entry_Ext[IPos] <> '?' ) THEN
            Match := Match AND ( FExt[IPos] = Entry_Ext[IPos] );
      UNTIL ( NOT Match ) OR ( IPos = 3 );

   Entry_Matches := Match;

END   (* Entry_Matches *);
